Package: sitadela
Type: Package
Title: An R package for the easy provision of simple but complete
        tab-delimited genomic annotation from a variety of sources and
        organisms
Authors@R: c(person(given="Panagiotis", family="Moulos",
    email="moulos@fleming.gr", role=c("aut", "cre")))
Depends: R (>= 4.1.0)
Imports: Biobase, BiocGenerics, biomaRt, Biostrings, Seqinfo,
        GenomicFeatures, GenomicRanges, IRanges, methods, parallel,
        Rsamtools, RSQLite, rtracklayer, S4Vectors, tools, txdbmaker,
        utils
Suggests: GenomeInfoDb, BiocStyle, BSgenome, knitr, rmarkdown, RMySQL,
        RUnit
Description: Provides an interface to build a unified database of
    genomic annotations and their coordinates (gene, transcript
    and exon levels). It is aimed to be used when simple 
    tab-delimited annotations (or simple GRanges objects) are 
    required instead of the more complex annotation Bioconductor 
    packages. Also useful when combinatorial annotation elements
    are reuired, such as RefSeq coordinates with Ensembl biotypes.
    Finally, it can download, construct and handle annotations with
    versioned genes and transcripts (where available, e.g. RefSeq
    and latest Ensembl). This is particularly useful in precision
    medicine applications where the latter must be reported.
License: Artistic-2.0
Encoding: UTF-8
URL: https://github.com/pmoulos/sitadela
biocViews: Software, WorkflowStep, RNASeq, Transcription, Sequencing,
        Transcriptomics, BiomedicalInformatics, FunctionalGenomics,
        SystemsBiology, AlternativeSplicing, DataImport, ChIPSeq
VignetteBuilder: knitr
BugReports: https://github.com/pmoulos/sitadela/issues
Version: 1.19.0
Date: 2025-07-22
Collate: 'argcheck.R' 'fromtxdb.R' 'query-ensembl.R' 'query-ncbi.R'
        'query-refseq.R' 'query-ucsc.R' 'query-utils.R' 'reduceops.R'
        'sitadela.R' 'testfuns.R' 'zzz.R'
git_url: https://git.bioconductor.org/packages/sitadela
git_branch: devel
git_last_commit: 45b2af3
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-11-04
NeedsCompilation: no
Packaged: 2025-11-05 01:04:18 UTC; biocbuild
Author: Panagiotis Moulos [aut, cre]
Maintainer: Panagiotis Moulos <moulos@fleming.gr>
