import(Biobase)
import(biomaRt)
import(GenomicRanges)
import(methods)
import(parallel)
import(Rsamtools)
import(RSQLite)

importFrom("BiocGenerics",match)
importFrom("BiocGenerics",unlist)
importFrom("BiocGenerics",which)

importFrom("Biostrings",alphabetFrequency)

importFrom("Seqinfo",seqinfo)
importFrom("Seqinfo",Seqinfo)
importFrom("Seqinfo",seqlevels)
importFrom("Seqinfo",seqlengths)
importFrom("Seqinfo","seqlengths<-")

importFrom("txdbmaker",makeTxDbFromGFF)
#importFrom("GenomicFeatures",makeTxDbFromGFF)
importFrom("GenomicFeatures",exons)
importFrom("GenomicFeatures",exonsBy)
importFrom("GenomicFeatures",genes)
importFrom("GenomicFeatures","seqlevels<-")
importFrom("GenomicFeatures",transcripts)
importFrom("GenomicFeatures",threeUTRsByTranscript)

importFrom("IRanges",IRanges)
importFrom("IRanges",slice)

importFrom("S4Vectors",DataFrame)
importFrom("S4Vectors",Rle)
importFrom("S4Vectors",queryHits)
importFrom("S4Vectors",runValue)
importFrom("S4Vectors",subjectHits)

importFrom("rtracklayer",import)

importFrom("tools",R_user_dir)

importFrom("utils",download.file)
importFrom("utils",head)
importFrom("utils","menu")
importFrom("utils",packageVersion)
importFrom("utils",read.delim)
importFrom("utils",read.table)
importFrom("utils",write.table)
importFrom("utils",zip)

export(addAnnotation)
export(addCustomAnnotation)
export(getAnnotation)
export(getDbPath)
export(getInstalledAnnotations)
export(getSeqInfo)
export(getUserAnnotations)
export(importCustomAnnotation)
export(loadAnnotation)
export(removeAnnotation)
export(setDbPath)
export(testCustomBuild)
export(testCustomGtf)
export(testKnownBuild)
export(testEnsembl)
export(testEnsemblSimple)
export(testUcsc)
export(testUcscAll)
export(testUcscUtr)
export(testUcscUtrAll)

