Apart from an artificial GTF file called "dummy.gtf.gz" which serves testing
of custom annotation building, there are three more examples of GTF files
accompanying the sitadela package. These are:

1. gadMor1_HE567025.gtf.gz and gadMor1_HE567025.txt.gz
A GTF file with the reference sequence HE567025 of the Atlantic cod (Gadus 
morhua) retrieved from UCSC, along with the length of the sequence, required
for a more complete custom annotation build.

2. dasNov3_JH569334.gtf.gz and dasNov3_JH569334.txt.gz
A GTF file with the reference sequence JH569334 of the Armadillo (Dasypus  
novemcinctus) retrieved from UCSC, along with the length of the sequence, 
required for a more complete custom annotation build.

3. dicLab1_LG3.gtf.gz and dicLab1_LG3.txt.gz
A GTF file with the reference sequence LG3 of the European bass (Dicentrarchus  
labrax) retrieved from Max Planck Institute for Molecular Genetics, Berlin
(http://seabass.mpipz.mpg.de/), along with the length of the sequence, required
for a more complete custom annotation build.

4. eboVir3_ncbiGene.gtf.gz and eboVir3_ncbiGene.txt.gz
A GTF file with the genome sequence of the EBOLA virus retrieved from UCSC, 
along with the length of the sequence, required for a more complete custom 
annotation build.
